@@ -8,7 +8,7 @@ use extended_primitives::Buffer;
 use futures::channel::mpsc::{UnboundedReceiver, UnboundedSender};
 use futures::SinkExt;
 use futures::StreamExt;
-use log::{debug, info, warn};
+use log::{debug, info, trace, warn};
 use rand::prelude::*;
 use serde::Serialize;
 use serde_json::json;
@@ -144,7 +144,7 @@ impl<State: Clone + Send + Sync + 'static> Connection<State> {
         //@todo could store this as a UUID type as well.
         let id = Uuid::new_v4();
 
-        info!("Accepting new miner. ID: {}", &id);
+        debug!("Accepting new miner. ID: {}", &id);
 
         let info = MinerInfo {
             ip: addr.ip(),
@@ -300,7 +300,7 @@ impl<State: Clone + Send + Sync + 'static> Connection<State> {
 
         let msg_string = serde_json::to_string(&message)?;
 
-        debug!("Sending message: {}", msg_string.clone());
+        trace!("Sending message: {}", msg_string.clone());
 
         let mut sender = self.sender.lock().await;
 
