@@ -1,11 +1,13 @@
 use crate::connection::Connection;
 use crate::router::Router;
 use crate::server::{UpstreamConfig, VarDiffConfig};
+use crate::types::MessageValue;
 use crate::BanManager;
 pub use crate::MinerList;
 use crate::{Error, Result};
 use async_std::net::TcpStream;
 use async_std::sync::Arc;
+use extended_primitives::Buffer;
 use futures::channel::mpsc::{unbounded, UnboundedReceiver};
 use futures::io::BufReader;
 use futures::io::ReadHalf;
@@ -137,8 +139,18 @@ pub async fn handle_connection<
                             Some(result) => match result {
                                 Ok((method, values)) => {
                                     if method == "result" {
-                                        //@todo handle bad sends
-                                        urtx.send(values).await;
+                                        match values {
+                                            //@todo if this is not StratumV1, we might not want to
+                                            //just continue (Although there are plenty of decent
+                                            //backstops here, just thinking super cautiously and
+                                            //covering all our basis. So let's review this and
+                                            //explcitly handle the scenario.)
+                                            MessageValue::StratumV1(map) => {
+                                                //@todo handle bad sends
+                                                urtx.send(map).await;
+                                            }
+                                            _ => {}
+                                        };
                                         continue;
                                     }
 
@@ -235,13 +247,131 @@ pub async fn handle_connection<
     }
 }
 
+pub enum MessageTypes {
+    RegisterWorker,
+    SubmitShare,
+    SubmitShareWithTime,
+    SubmitShareWithVersion,
+    SubmitShareWithTimeAndVersion,
+    UnregisterWorker,
+    MiningSetDiff,
+
+    Unknown(u8),
+}
+
+impl MessageTypes {
+    pub fn from_u8(cmd: u8) -> Self {
+        match cmd {
+            0x01 => MessageTypes::RegisterWorker,
+            0x02 => MessageTypes::SubmitShare,
+            0x03 => MessageTypes::SubmitShareWithTime,
+            0x04 => MessageTypes::UnregisterWorker,
+            0x05 => MessageTypes::MiningSetDiff,
+            //@note not sure why these are so far after the originals. Makes me think there are
+            //other messages we are missing here, but can figure that out later.
+            0x12 => MessageTypes::SubmitShareWithVersion,
+            0x13 => MessageTypes::SubmitShareWithTimeAndVersion,
+            _ => MessageTypes::Unknown(cmd),
+        }
+    }
+
+    pub fn to_string(&self) -> String {
+        match *self {
+            MessageTypes::RegisterWorker => String::from("exMessageRegisterWorker"),
+            MessageTypes::SubmitShare => String::from("exMessageSubmitShare"),
+            MessageTypes::SubmitShareWithTime => String::from("exMessageSubmitShareWithTime"),
+            MessageTypes::UnregisterWorker => String::from("exMessageUnregisterWorker"),
+            MessageTypes::MiningSetDiff => String::from("exMessageMiningSetDiff"),
+            MessageTypes::SubmitShareWithVersion => String::from("exMessageSubmitShareWithVersion"),
+            MessageTypes::SubmitShareWithTimeAndVersion => {
+                String::from("exMessageSubmitShareWithTimeAndVersion")
+            }
+            //@todo I'd actually prefer we throw an error here but we can come back to that later
+            //as well.
+            //But seems important for security otherwise someone could just hit us with fake Ex
+            //messages all day.
+            _ => String::from(""),
+        }
+    }
+}
+
+// ex-message:
+//   magic_number	uint8_t		magic number for Ex-Message, always 0x7F
+//   type/cmd		uint8_t		message type
+//   length			uint16_t	message length (include header self)
+//   message_body	uint8_t[]	message body
+//btc agent style messages.
+pub struct ExMessageGeneric {
+    pub magic_number: u8,
+    //@todo turn this into an Enum?
+    pub cmd: MessageTypes,
+    pub length: u16,
+    pub body: Buffer,
+}
+
+//@todo let's make sure we get some good errors in here for if this shit blows up.
+impl ExMessageGeneric {
+    //@todo actually just call this inside of that function.
+    pub fn from_buffer(buffer: &mut Buffer) -> Result<Self> {
+        let magic_number = buffer.read_u8().map_err(|_| Error::BrokenExHeader)?;
+        let cmd = buffer.read_u8().map_err(|_| Error::BrokenExHeader)?;
+        let length = buffer.read_u16().map_err(|_| Error::BrokenExHeader)?;
+
+        let body = buffer.clone();
+
+        let cmd = MessageTypes::from_u8(cmd);
+
+        // if cmd == MessageTypes::Unknown(_) {
+        //@todo custom error here pls.
+        // return Error::
+        // }
+
+        Ok(ExMessageGeneric {
+            magic_number,
+            cmd,
+            length,
+            body,
+        })
+    }
+}
+
+//@todo I think we are going to have to change the return method of this...
 pub async fn next_message(
     stream: &mut BufReader<ReadHalf<TcpStream>>,
-) -> Result<(String, serde_json::map::Map<String, serde_json::Value>)> {
+) -> Result<(String, MessageValue)> {
     //I don't actually think this has to loop here.
     loop {
         // let mut stream = self.read_half.lock().await;
 
+        //@todo let's try to revamp this before the push this live.
+        //@note this is probably super inefficient (Imagine transfering a full json message
+        //everytime just to do it again...)
+        let magic_number: u8 = 0x7F;
+        let peak = stream.fill_buf().await?;
+
+        if peak.len() == 0 {
+            // self.shutdown().await?;
+            return Err(Error::StreamClosed);
+        }
+
+        if peak[0] == magic_number {
+            //@handle ex message here.
+            let mut buffer = Buffer::from(peak);
+            //@todo I think we need a consume here... But we need to be very careful about this
+            //placement.
+            stream.consume_unpin(buffer.len());
+
+            let ex_message = ExMessageGeneric::from_buffer(&mut buffer)?;
+            return Ok((
+                ex_message.cmd.to_string(),
+                MessageValue::ExMessage(ex_message.body.clone()),
+            ));
+        }
+
+        //If we have reached here, then we did not breat the "Peak test" searching for the magic
+        //number of ExMessage.
+
+        //@todo let's break this into 2 separate functions eh?
         let mut buf = String::new();
         let num_bytes = stream.read_line(&mut buf).await?;
 
@@ -291,7 +421,7 @@ pub async fn next_message(
                 //We only mark them as active if the message/method was valid
                 // self.stats.lock().await.last_active = Utc::now().naive_utc();
 
-                return Ok((method_string.to_owned(), msg));
+                return Ok((method_string.to_owned(), MessageValue::StratumV1(msg)));
             } else {
                 //@todo improper format
                 return Err(Error::MethodDoesntExist);
