@@ -1,4 +1,5 @@
 use async_std::sync::{Arc, Mutex};
+use extended_primitives::Buffer;
 use serde::{Deserialize, Serialize};
 use std::fmt;
 
@@ -40,3 +41,8 @@ impl std::fmt::Display for ID {
         }
     }
 }
+
+pub enum MessageValue {
+    StratumV1(serde_json::map::Map<String, serde_json::Value>),
+    ExMessage(Buffer),
+}
